import "./alert.scss";
import { disableBodyScroll, enableBodyScroll } from "body-scroll-lock";
import { globalDefaultsManager } from "../initializeLibrary.js";
import mergeConfig from "../utils/mergeConfig.js";
import colorSchemeHandler from "../utils/colorSchemeHandler.js";

export class Alert {
    constructor(text, actions = [], config = {}) {
        this.text = text;
        this.actions = actions ?? [];

        let defaultConfig = {
            description: null,

            ...globalDefaultsManager.defaults.alerts,
            // these options have been externalized into defaults
            // doBSL, throw, direction, dismissDescription, dismissStyle, icon, theme
        };

        this.config = mergeConfig(defaultConfig, config);

        this.element = null;
        this.initializeHtml();
    }

    initializeHtml() {
        let container = document.createElement("div");
        container.classList.add("hm-alert");

        // Add color scheme handler
        colorSchemeHandler(container, this.config.theme);

        // Icon
        if (this.config.icon.html) {
            let iconContainer = document.createElement("div");
            iconContainer.classList.add("hm-icon-container");
            iconContainer.classList.add(`hm-${this.config.icon.style ?? "blue"}`);
            iconContainer.innerHTML = this.config.icon.html;

            container.appendChild(iconContainer);
        }

        // Text Container
        let textContainer = document.createElement("div");
        textContainer.classList.add("hm-text-container");
        container.appendChild(textContainer);

        // Text
        let text = document.createElement("p");
        text.classList.add("hm-text");
        text.innerHTML = this.text;
        textContainer.appendChild(text);

        // Description
        if (this.config.description) {
            let description = document.createElement("p");
            description.classList.add("hm-description");
            description.innerHTML = this.config.description;
            textContainer.appendChild(description);
        }

        // Actions container
        let actionsContainer = document.createElement("div");
        actionsContainer.classList.add("hm-actions");
        container.appendChild(actionsContainer);

        if (this.config.direction == "vertical") actionsContainer.classList.add("hm-vertical");
        else actionsContainer.classList.add("hm-horizontal");

        // Promise
        let promise = new Promise((resolve, reject) => {
            let resolve_ = (arg) => {
                this.close();
                resolve(arg);
            };

            let reject_ = (arg) => {
                this.close();
                reject(arg);
            };

            // Actions
            this.actions?.forEach((action) => {
                if (!action.text || !action.handler) return;

                let btn = document.createElement("button");
                btn.innerHTML = action.text;
                btn.classList.add("hm-button");
                btn.classList.add(`hm-${action.style ?? "blue"}`);

                btn.addEventListener("click", () => action.handler(resolve_, reject_, this.close));

                actionsContainer.appendChild(btn);
            });

            // Close button
            let closeBtn = document.createElement("button");
            closeBtn.classList.add("hm-button");
            closeBtn.classList.add(`hm-${this.config.dismissStyle}`);
            closeBtn.innerHTML = this.config.dismissDescription;

            closeBtn.addEventListener("click", () => {
                this.close();

                if (this.config.throw == true) reject();
                else resolve(false);
            });

            actionsContainer.appendChild(closeBtn);
        });

        // Finish
        this.element = container;
        this.promise = promise;
    }

    show() {
        this.element.classList.add("hm-alert-show");
        if (this.config.doBSL != false) disableBodyScroll(this.element);

        // Darken background
        document.querySelector(".hm-alerts-container").classList.add("hm-darken");

        // Show element
        document.querySelector(".hm-alerts-container").appendChild(this.element);

        // Remove show animation after playing it
        setTimeout(() => {
            this.element.classList.remove("hm-alert-show");
        }, 400);
    }

    close() {
        this.element.classList.add("hm-alert-close");
        if (this.config.doBSL != false) enableBodyScroll(this.element);

        // If this is the last alert, then remove the darker background
        if (document.querySelector(".hm-alerts-container").childNodes.length - 1 <= 0) {
            document.querySelector(".hm-alerts-container").classList.remove("hm-darken");
        }

        // Remove after playing exit animation
        setTimeout(() => {
            document.querySelector(".hm-alerts-container").removeChild(this.element);
        }, 400);
    }
}

export default function alert(text, actions = [], config = {}) {
    let alert = new Alert(text, actions, config);
    alert.show();

    return alert.promise;
}
