import "./snackbar.scss";
import { globalDefaultsManager } from "../initializeLibrary.js";
import mergeConfig from "../utils/mergeConfig.js";
import colorSchemeHandler from "../utils/colorSchemeHandler.js";

export class Snackbar {
    constructor(text, actions = [], config = {}) {
        this.text = text;
        this.actions = actions ?? [];

        let defaultConfig = {
            description: null,

            ...globalDefaultsManager.defaults.snackbars,
            // these options have been externalized into defaults
            // duration, autoClose, throw, dismissDescription, dismissStyle, icon, theme
        };

        this.config = mergeConfig(defaultConfig, config);

        this.element = null;
        this.timeouts = [];
        this.initializeHtml();
    }

    initializeHtml() {
        let container = document.createElement("div");
        container.classList.add("hm-snackbar");
        container.classList.add("hm-snackbar-show");

        // Add color scheme handler
        colorSchemeHandler(container, this.config.theme);

        // Icon
        if (this.config.icon.html) {
            let iconContainer = document.createElement("div");
            iconContainer.classList.add("hm-icon-container");
            iconContainer.classList.add(`hm-${this.config.icon.style ?? "blue"}`);
            iconContainer.innerHTML = this.config.icon.html;

            container.appendChild(iconContainer);
        }

        // Text Container
        let textContainer = document.createElement("div");
        textContainer.classList.add("hm-text-container");
        container.appendChild(textContainer);

        // Text
        let text = document.createElement("p");
        text.classList.add("hm-text");
        text.innerHTML = this.text;
        textContainer.appendChild(text);

        // Description
        if (this.config.description) {
            let description = document.createElement("p");
            description.classList.add("hm-description");
            description.innerHTML = this.config.description;
            textContainer.appendChild(description);
        }

        // Promise
        let promise = new Promise((resolve, reject) => {
            let resolve_ = (arg) => {
                this.close();
                resolve(arg);
            };

            let reject_ = (arg) => {
                this.close();
                reject(arg);
            };

            // actions
            this.actions?.forEach((action) => {
                if (!action.text || !action.handler) return;

                let btn = document.createElement("button");
                btn.innerHTML = action.text;
                btn.classList.add("hm-button");
                btn.classList.add(`hm-${action.style ?? "secondary"}`);

                btn.addEventListener("click", () => action.handler(resolve_, reject_, this.close));

                container.appendChild(btn);
            });

            // Close button
            let closeBtn = document.createElement("button");
            closeBtn.classList.add("hm-button");
            closeBtn.classList.add(`hm-${this.config.dismissStyle}`);

            if (this.config.dismissDescription) closeBtn.innerHTML = this.config.dismissDescription;
            else
                closeBtn.innerHTML = `
            <svg xmlns="http://www.w3.org/2000/svg" version="1.1" viewBox="0 0 1000 1000" width="17" height="17">
                <path d="M150.062+74.4688C130.868+74.4544+111.656+81.7726+97+96.4062C67.6887+125.674+67.6701+173.158+96.9375+202.469L394.031+500L96.9375+797.531C67.6701+826.843+67.6887+874.326+97+903.594C126.311+932.861+173.795+932.843+203.062+903.531L500+606.156L796.938+903.531C826.205+932.843+873.689+932.862+903+903.594C932.311+874.326+932.33+826.842+903.062+797.531L605.969+500L903.062+202.469C932.33+173.157+932.311+125.674+903+96.4062C888.344+81.7726+869.132+74.4544+849.938+74.4688C830.743+74.4831+811.571+81.8131+796.938+96.4688L500+393.844L203.062+96.4688C188.429+81.8131+169.257+74.4831+150.062+74.4688Z" />
            </svg>`;

            closeBtn.addEventListener("click", () => {
                this.close();
                if (this.config.throw == true) reject();
                else resolve(false);
            });

            container.appendChild(closeBtn);
        });

        // Finish
        this.element = container;
        this.promise = promise;
    }

    show() {
        // Insert element
        let snackbarsContainer = document.querySelector(".hm-snackbars-container");
        snackbarsContainer.insertBefore(this.element, snackbarsContainer.firstChild);

        // Setup auto-closing after duration is over
        if (this.config.autoClose == true) this.timeouts.push(setTimeout(() => this.close(), this.config.duration));

        // Remove show animation after playing it
        this.timeouts.push(
            setTimeout(() => {
                this.element.classList.remove("hm-snackbar-show");
            }, 400)
        );
    }

    close() {
        this.element.classList.add("hm-snackbar-close");

        // Remove timeouts
        this.timeouts.forEach((timeout) => {
            clearTimeout(timeout);
        });

        // Remove after playing exit animation
        setTimeout(() => {
            document.querySelector(".hm-snackbars-container").removeChild(this.element);
        }, 300);
    }
}

export default function snackbar(text, actions = [], config = {}) {
    let snackbar = new Snackbar(text, actions, config);
    snackbar.show();

    return snackbar.promise;
}
